#pragma once

#include <QGeoCoordinate>
#include <QSharedPointer>
#include <QThread>

#include "geometry/snake.h"
#include <atomic>
#include <condition_variable>
#include <functional>
#include <mutex>

struct RoutingData {
  snake::Transects transects;
  std::vector<snake::Solution> solutionVector;
  std::string errorString;
};

struct RoutingParameter {
  RoutingParameter() : numSolutions(1), numRuns(1) {}
  snake::FPolygon safeArea;
  std::size_t numSolutions;
  std::size_t numRuns;
};
//!
//! \brief The CSWorker class
//! \note Don't call QThread::start, QThread::quit, etc. onyl use Worker
//! members!
class RoutingThread : public QThread {
  Q_OBJECT
  using Lock = std::unique_lock<std::mutex>;

public:
  using PtrRoutingData = shared_ptr<RoutingData>;
  using Generator = std::function<bool(snake::Transects &)>;
  using Consumer = std::function<void(const RoutingData &)>;

  RoutingThread(QObject *parent = nullptr);
  ~RoutingThread() override;

  bool calculating() const;

public slots:
  void route(const RoutingParameter &par, const Generator &generator);

signals:
  void result(PtrRoutingData pTransects);
  void calculatingChanged();

protected:
  void run() override;

private:
  mutable std::mutex _mutex;
  mutable std::condition_variable _cv;
  // Internal data
  RoutingParameter _par;
  Generator _generator; // transect generator
  // State
  std::atomic_bool _calculating;
  std::atomic_bool _stop;
  std::atomic_bool _restart;
};
